#! /usr/bin/env python3
"""
This is a Bootstrap script for wrapper.py, in order to retain compatibility with earlier LibreNMS setups
"""

import logging
import os
import sys
from argparse import ArgumentParser

import LibreNMS
import LibreNMS.wrapper as wrapper

WRAPPER_TYPE = "discovery"
DEFAULT_WORKERS = 1

"""
    Take the amount of threads we want to run in parallel from the commandline
    if None are given or the argument was garbage, fall back to default
"""
usage = (
    "usage: %(prog)s [options] <amount_of_workers> (Default: {}"
    "(Do not set too high, or you will get an OOM)".format(DEFAULT_WORKERS)
)
description = "Spawn multiple discovery processes in parallel."
parser = ArgumentParser(usage=usage, description=description)
parser.add_argument(dest="amount_of_workers", default=DEFAULT_WORKERS)
parser.add_argument(
    "-d",
    "--debug",
    dest="debug",
    action="store_true",
    default=False,
    help="Enable debug output. WARNING: Leaving this enabled will consume a lot of disk space.",
)
parser.add_argument(
    "-m",
    "--modules",
    dest="modules",
    default="",
    help="Enable passing of a module string, modules are separated by comma",
)
args = parser.parse_args()

config = LibreNMS.get_config_data(os.path.dirname(os.path.realpath(__file__)))
if not config:
    logger = logging.getLogger(__name__)
    logger.critical("Could not run {} wrapper. Missing config".format(WRAPPER_TYPE))
    sys.exit(1)
log_dir = config["log_dir"]
log_file = os.path.join(log_dir, WRAPPER_TYPE + "_wrapper.log")
logger = LibreNMS.logger_get_logger(log_file, debug=args.debug)

scheduler = config.get("schedule_type").get("discovery", "legacy")
enabled = True if scheduler == "legacy" else scheduler == "cron"
if not enabled:
    logger.debug("Discovery is not enabled for cron scheduling")
    sys.exit(0)

try:
    amount_of_workers = int(args.amount_of_workers)
except (IndexError, ValueError):
    amount_of_workers = DEFAULT_WORKERS
    logger.warning(
        "Bogus number of workers given. Using default number ({}) of workers.".format(
            amount_of_workers
        )
    )

wrapper.wrapper(
    WRAPPER_TYPE,
    amount_of_workers=amount_of_workers,
    config=config,
    log_dir=log_dir,
    modules=args.modules or "",
    _debug=args.debug,
)



import os


class DBConfig:
    """
    Bare minimal config class for LibreNMS.DB class usage
    """

    # Start with defaults and override
    db_host = "localhost"
    db_port = 0
    db_socket = None
    db_user = "librenms"
    db_pass = ""
    db_name = "librenms"
    db_sslmode = "disabled"
    db_ssl_ca = "/etc/ssl/certs/ca-certificates.crt"

    def populate(self, _config):
        self.db_host = os.getenv("DB_HOST", _config.get("db_host", self.db_host))
        self.db_name = os.getenv("DB_DATABASE", _config.get("db_name", self.db_name))
        self.db_pass = os.getenv("DB_PASSWORD", _config.get("db_pass", self.db_pass))
        self.db_port = int(os.getenv("DB_PORT", _config.get("db_port", self.db_port)))
        self.db_socket = os.getenv(
            "DB_SOCKET", _config.get("db_socket", self.db_socket)
        )
        self.db_user = os.getenv("DB_USERNAME", _config.get("db_user", self.db_user))
        self.db_sslmode = os.getenv(
            "DB_SSLMODE", _config.get("db_sslmode", self.db_sslmode)
        )
        self.db_ssl_ca = os.getenv(
            "MYSQL_ATTR_SSL_CA", _config.get("db_ssl_ca", self.db_ssl_ca)
        )



#! /usr/bin/env python3
import pkg_resources
import sys
from pkg_resources import DistributionNotFound, VersionConflict

args = sys.argv

# verbose flag
verbose = "-v" in args

requirements = ["PyMySQL"]

try:
    pkg_resources.require(requirements)
except DistributionNotFound as req:
    if verbose:
        print(req)
    sys.exit(1)
except VersionConflict as req:
    if verbose:
        print(req)
    sys.exit(2)
sys.exit(0)



#!/usr/bin/env python3

import argparse
import logging
import os
import sys
import threading
from logging import info

import LibreNMS

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="LibreNMS Service - manages polling and other periodic processes"
    )
    parser.add_argument(
        "-g",
        "--group",
        nargs="+",
        type=int,
        help="Set the poller group for this poller",
    )
    parser.add_argument("-v", "--verbose", action="count", help="Show verbose output.")
    parser.add_argument("-d", "--debug", action="store_true", help="Show debug output.")
    parser.add_argument(
        "-o",
        "--log-output",
        nargs="?",
        const="file",
        default="none",
        type=LibreNMS.LogOutput,
        choices=list(LibreNMS.LogOutput),
        metavar="DEST",
        help=(
            "Where to direct poller log messages.\n"
            "  --log-output         = log to file (same as --log-output file)\n"
            "  --log-output DEST    = log to none, passthrough, logger, file\n"
            "  (not specified)      = no dedicated poller output logging (=none)"
        ),
    )
    parser.add_argument(
        "-m",
        "--multiple",
        action="store_true",
        help="Allow multiple instances of the service.",
    )
    parser.add_argument(
        "-t",
        "--timestamps",
        action="store_true",
        help="Include timestamps in the logs (not normally needed for syslog/journald",
    )

    args = parser.parse_args()

    if args.timestamps:
        logging.basicConfig(
            format="%(asctime)s %(threadName)s(%(levelname)s):%(message)s"
        )
    else:
        logging.basicConfig(format="%(threadName)s(%(levelname)s):%(message)s")

    if args.verbose:
        logging.getLogger().setLevel(logging.INFO)
    elif args.debug:
        logging.getLogger().setLevel(logging.DEBUG)
    else:
        logging.getLogger().setLevel(logging.WARNING)

    info("Configuring LibreNMS service")
    try:
        service = LibreNMS.Service()
    except Exception as e:
        # catch any initialization errors and print the message instead of a stack trace
        print(e)
        sys.exit(2)

    service.config.single_instance = args.multiple
    service.config.log_output = args.log_output

    if args.group:
        if isinstance(args.group, list):
            service.config.group = args.group
        else:
            service.config.group = [args.group]

    info(
        "Entering main LibreNMS service loop on {}/{}...".format(
            os.getpid(), threading.current_thread().name
        )
    )
    service.start()
